package de.towerdefence.server.server.channels.match;

import de.towerdefence.server.match.MatchService;
import de.towerdefence.server.server.JsonWebsocketHandler;
import de.towerdefence.server.session.Channel;
import de.towerdefence.server.session.SessionsService;
import org.springframework.web.socket.WebSocketSession;

import java.io.IOException;
import java.util.Map;
import java.util.concurrent.*;

public class MatchWebsocketHandler extends JsonWebsocketHandler {

    final Map<WebSocketSession, ScheduledFuture<?>> sessionTaskMap = new ConcurrentHashMap<>();
    private static final ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(1);
    private final MatchService matchService;

    public MatchWebsocketHandler(SessionsService sessionsService, MatchService matchService) {
        super(Channel.MATCH, sessionsService);
        this.matchService = matchService;
    }

    @Override
    public void afterConnectionEstablished(WebSocketSession session) {
        super.afterConnectionEstablished(session);
        ScheduledFuture<?> scheduledTask = scheduler.scheduleAtFixedRate(
            () -> sendCurrentTime(session),
            0,
            1,
            TimeUnit.SECONDS
        );
        sessionTaskMap.put(session, scheduledTask);
    }

    private void sendCurrentTime(WebSocketSession session) {
        ScheduledFuture<?> task = sessionTaskMap.get(session);
        try {
            if (!session.isOpen()) {
                throw new RuntimeException("Session is not open");
            }
            new TimeMessage(
                System.currentTimeMillis(),
                matchService.get(this.sessionPlayers.get(session)).getMatchId()
            ).send(session);
        } catch (RuntimeException | IOException e) {
            task.cancel(true);
            sessionTaskMap.remove(session);
        }
    }
}
